@@ -28,7 +28,6 @@
  * Key properties are:
  * - @link keyname Key name @endlink
  * - @link keyvalue Key value @endlink
- * - @link keySetType() Data type @endlink
  * - @link keyGetComment() Key comment @endlink
  * - @link keyGetOwner() Key owner @endlink
  * - @link keymeta UID, GID and filesystem-like mode permissions @endlink
@@ -40,8 +39,8 @@
 
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #if DEBUG && HAVE_STDIO_H
@@ -145,12 +144,6 @@ keyDel (emptyNamedKey);
  *   Next parameter is the owner. See keySetOwner().
  * - keyswitch_t::KEY_COMMENT \n
  *   Next parameter is a comment. See keySetComment().
- * - keyswitch_t::KEY_REMOVE \n
- *   Mark the key to be removed instead of set it.
- *   See keyRemove().
- * - keyswitch_t::KEY_STAT \n
- *   Mark the key to be stated instead of get it.
- *   See keyStat().
  * - keyswitch_t::KEY_END \n
  *   Must be the last parameter passed to keyNew(). It is always
  *   required, unless the @p keyName is 0.
@@ -273,10 +266,10 @@ Key *keyNew(const char *keyName, ...) {
 Key *keyVNew (const char *keyName, va_list va)
 {
 	Key *key=0;
-	type_t type=0;
 	keyswitch_t action=0;
 	void * value=0;
 	ssize_t valueSize=-1;
+	void (*p) (void)=0;
 
 	key=(Key *)malloc(sizeof(Key));
 	if (!key) return 0;
@@ -288,20 +281,19 @@ Key *keyVNew (const char *keyName, va_list va)
 		action=va_arg(va, keyswitch_t);
 		while (action) {
 			switch (action) {
-				case KEY_TYPE:
-					type = (type_t) va_arg(va, type_t);
-					keySetType(key, type);
-					break;
 				case KEY_SIZE:
 					valueSize=va_arg(va, size_t);
 					break;
+				case KEY_BINARY:
+					keySetMeta (key, "binary", "");
+					break;
 				case KEY_VALUE:
 					value = va_arg(va, void *);
 					if (valueSize>=0 && keyIsBinary(key))
 					{
 						keySetBinary(key,value, valueSize);
 					} else if (keyIsBinary(key)) {
-						valueSize = (ssize_t) kdbiStrLen (value);
+						valueSize = (ssize_t) elektraStrLen (value);
 						keySetBinary(key,value, valueSize);
 					} else {
 						keySetString(key,value);
@@ -322,15 +314,18 @@ Key *keyVNew (const char *keyName, va_list va)
 				case KEY_COMMENT:
 					keySetComment(key,va_arg(va,char *));
 					break;
-				case KEY_REMOVE:
-					keyRemove(key);
-					break;
-				case KEY_STAT:
-					keyStat(key);
-					break;
 				case KEY_DIR:
 					keySetDir(key);
 					break;
+				case KEY_FUNC:
+					p = va_arg(va, void(*)(void));
+					keySetBinary(key, &p, sizeof(p));
+					break;
+				case KEY_META:
+					value = va_arg (va,char *);
+					/*First parameter is name*/
+					keySetMeta (key, value, va_arg(va,char *));
+					break;
 				default:
 #if DEBUG
 					fprintf (stderr, "Unknown option in keyNew %ld\n", (long int)action);
@@ -405,16 +400,17 @@ Key* keyDup(const Key *source)
 
 	/* prepare to set dynamic properties */
 	dest->key=
-	dest->comment=
-	dest->owner=
-	dest->data=0;
+	dest->data.v=
+	dest->meta=0;
 
 	dest->ksReference = 0;
 
 	if (source->key && keySetName(dest,source->key) == -1) goto memerror;
-	if (source->comment && keySetComment(dest,source->comment) == -1) goto memerror;
-	if (source->owner && keySetOwner(dest,source->owner) == -1) goto memerror;
-	if (source->data && keySetRaw(dest,source->data,source->dataSize) == -1) goto memerror;
+	if (source->data.v && keySetRaw(dest,source->data.v,source->dataSize) == -1) goto memerror;
+	if (source->meta)
+	{
+		dest->meta = ksDup (source->meta);
+	}
 	
 	return dest;
 memerror:
@@ -436,7 +432,7 @@ memerror:
  * you can do so:
  *
  * @code
-int h (Key *k)
+void h (Key *k)
 {
 	// receive key c
 	keyCopy (k, c);
@@ -453,13 +449,47 @@ int h (Key *k)
  * you get a fresh dest key.
  *
  * @code
-int g (Key *k)
+void g (Key *k)
 {
 	keyCopy (k, 0);
 	// k is now an empty and fresh key
 }
  * @endcode
  *
+ * The meta data will be duplicated for the destination
+ * key. So it will not take much additional space, even
+ * with lots of metadata.
+ *
+ * If you want to copy all metadata, but keep the old
+ * value you can use keyCopy() too.
+ *
+ * @code
+void j (Key *k)
+{
+	size_t size = keyGetValueSize (k);
+	char *value = malloc (size);
+	int bstring = keyIsString (k);
+
+	// receive key c
+	memcpy (value, keyValue(k), size);
+	keyCopy (k, c);
+	if (bstring) keySetString (k, value);
+	else keySetBinary (k, value, size);
+	free (value);
+	// the caller will see the changed key k
+	// with the metadata from c
+}
+ * @endcode
+ *
+ * @note Next to the value itself we also need to remember
+ *       if the value was string or binary. So in fact the
+ *       meta data of the resulting key k in that
+ *       example is not a complete
+ *       duplicate, because the meta data "binary" may
+ *       differ. Similar considerations might be necessary
+ *       for the type of the key and so on, depending on the
+ *       concrete situation.
+ *
  * @param dest the key which will be written to
  * @param source the key which should be copied
  *     or NULL to clean the destination key
@@ -496,14 +526,15 @@ int keyCopy (Key *dest, const Key *source)
 
 	/* prepare to set dynamic properties */
 	dest->key=
-	dest->comment=
-	dest->owner=
-	dest->data=0;
+	dest->data.v=
+	dest->meta=0;
 
 	if (keySetName(dest,source->key) == -1) return -1;
-	if (keySetComment(dest,source->comment) == -1) return -1;
-	if (keySetOwner(dest,source->owner) == -1) return -1;
-	if (keySetRaw(dest,source->data,source->dataSize) == -1) return -1;
+	if (keySetRaw(dest,source->data.v,source->dataSize) == -1) return -1;
+	if (source->meta)
+	{
+		dest->meta = ksDup (source->meta);
+	}
 
 	return 1;
 }
@@ -545,7 +576,7 @@ int keyDel(Key *key) {
 	}
 
 	rc=keyClear(key);
-	kdbiFree (key);
+	elektraFree (key);
 
 	return rc;
 }
@@ -574,9 +605,8 @@ int f (Key *k)
 int keyClear(Key *key)
 {
 	if (key->key) free(key->key);
-	if (key->data) free(key->data);
-	if (key->comment) free(key->comment);
-	if (key->owner) free(key->owner);
+	if (key->data.v) free(key->data.v);
+	if (key->meta) ksDel(key->meta);
 
 	memset(key,0,sizeof(Key));
 	keyInit (key);
@@ -608,6 +638,8 @@ keyDel (k); // now really free it
  * nothing will happen and SSIZE_MAX will be
  * returned.
  *
+ * @note keyDup() will reset the references for dupped key.
+ *
  * @return the value of the new reference counter
  * @return -1 on null pointer
  * @return SSIZE_MAX when maximum exceeded
@@ -628,11 +660,19 @@ ssize_t keyIncRef(Key *key)
 /**
  * Decrement the viability of a key object.
  *
+ * The references will be decremented for ksPop() or successful calls
+ * of ksLookup() with the option KDB_O_POP.
+ * It will also be decremented with an following keyDel() in
+ * the case that an old key is replaced with another key with
+ * the same name.
+ *
  * The reference counter can't be decremented
  * once it reached 0. In that situation
  * nothing will happen and 0 will be
  * returned.
  *
+ * @note keyDup() will reset the references for dupped key.
+ *
  * @return the value of the new reference counter
  * @return -1 on null pointer
  * @return 0 when the key is ready to be freed
@@ -654,15 +694,13 @@ ssize_t keyDecRef(Key *key)
 /**
  * Return how many references the key has.
  *
- * The references will be incremented when ksAppendKey()
- * or ksAppend() uses the key and will be decremented when
- * ksPop() is used.
+ * The references will be incremented on successful calls to
+ * ksAppendKey() or ksAppend().
  *
- * keyDup() will reset the references for dupped
- * key.
+ * @note keyDup() will reset the references for dupped key.
  *
  * For your own applications you can use
- * keyIncRef() and keyDelRef() for reference
+ * keyIncRef() and keyDecRef() for reference
  * counting. Keys with zero references
  * will be deleted when using keyDel().
  *
