@@ -70,8 +70,8 @@ ksDel (myConfig); // delete keyset and all keys appended
  * @{
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #if DEBUG && HAVE_STDIO_H
@@ -90,7 +90,7 @@ ksDel (myConfig); // delete keyset and all keys appended
 #include <string.h>
 #endif
 
-#include "kdbbackend.h"
+#include "kdbinternal.h"
 
 
 
@@ -117,9 +117,15 @@ ksDel (keys);
  * read the next statements.
  *
  * If you want a keyset with length 15 (because you know of your
- * application that you normally need about 12 up to 14 keys), use:
+ * application that you normally need about 12 up to 15 keys), use:
  * @code
-KeySet * keys = ksNew (15, KS_END);
+KeySet * keys = ksNew (15,
+	keyNew ("user/sw/app/fixedConfiguration/key01", KEY_SWITCH_VALUE, "value01", 0),
+	keyNew ("user/sw/app/fixedConfiguration/key02", KEY_SWITCH_VALUE, "value02", 0),
+	keyNew ("user/sw/app/fixedConfiguration/key03", KEY_SWITCH_VALUE, "value03", 0),
+	// ...
+	keyNew ("user/sw/app/fixedConfiguration/key15", KEY_SWITCH_VALUE, "value15", 0),
+	KS_END);
 // work with it
 ksDel (keys);
  * @endcode
@@ -169,7 +175,7 @@ KeySet *ksVNew (size_t alloc, va_list va)
 	KeySet *keyset=0;
 	Key *key = 0;
 
-	keyset= (KeySet *)kdbiMalloc(sizeof(KeySet));
+	keyset= (KeySet *)elektraMalloc(sizeof(KeySet));
 	if (!keyset)
 	{
 		/*errno = KDB_ERR_NOMEM;*/
@@ -177,10 +183,11 @@ KeySet *ksVNew (size_t alloc, va_list va)
 	}
 	ksInit(keyset);
 
+	alloc ++; /* for ending null byte */
 	if (alloc < KEYSET_SIZE) keyset->alloc=KEYSET_SIZE;
 	else keyset->alloc=alloc;
 
-	keyset->array = kdbiMalloc (sizeof(struct _Key *) * keyset->alloc);
+	keyset->array = elektraMalloc (sizeof(struct _Key *) * keyset->alloc);
 	if (!keyset->array)
 	{
 		/*errno = KDB_ERR_NOMEM;*/
@@ -189,7 +196,7 @@ KeySet *ksVNew (size_t alloc, va_list va)
 	keyset->array[0] = 0;
 	
 
-	if (alloc) {
+	if (alloc != 1) {
 		key = (struct _Key *) va_arg (va, struct _Key *);
 		while (key) {
 			ksAppendKey(keyset, key);
@@ -305,7 +312,7 @@ int ksDel(KeySet *ks)
 	if (!ks) return -1;
 
 	rc=ksClose(ks);
-	kdbiFree(ks);
+	elektraFree(ks);
 
 	return rc;
 }
@@ -328,16 +335,16 @@ int ksClear(KeySet *ks)
 
 	if (ks->array)
 	{	/* go back to standard size KEYSET_SIZE */
-		if (kdbiRealloc ((void**) &ks->array, sizeof(struct _Key *) * KEYSET_SIZE) == -1)
+		if (elektraRealloc ((void**) &ks->array, sizeof(struct _Key *) * KEYSET_SIZE) == -1)
 		{
 			/*errno = KDB_ERR_NOMEM;*/
-			kdbiFree (ks->array);
+			elektraFree (ks->array);
 			ks->array = 0;
 			ks->size = 0;
 			return -1;
 		}
 	} else {
-		if ((ks->array = kdbiMalloc (sizeof(struct _Key *) * KEYSET_SIZE)) == 0)
+		if ((ks->array = elektraMalloc (sizeof(struct _Key *) * KEYSET_SIZE)) == 0)
 		{
 			/*errno = KDB_ERR_NOMEM;*/
 			ks->size = 0;
@@ -353,169 +360,147 @@ int ksClear(KeySet *ks)
 
 
 /*
- * Returns if keyset needs sort.
+ * Returns 0.
  *
- * It is very inefficient to resort the keyset every time when a
- * key or a keyset is appended, so only a flag will be set to
- * show that the keyset is not sorted any more.
+ * @deprecated dont use
  *
- * Before operations where the code depends on a sorted keyset,
- * namely in kdbGet(), kdbSet() and ksLookup(), this simple code
- * sorts when needed:
- *
- * @code
-if (ksNeedSort(ks)) ksSort(ks);
- * @endcode
- *
- * @warning ksNeedSort() does not track every change which actually
- * affects sorting: changing of names with keySetName() and
- * marking keys remove with keyRemove() won't be recognized.
- * But any ks* Method will leave the KeySet sorted or
- * set the flag.
- *
- * So if your code might rename keys or flag them to remove,
- * make sure that you use
- *
- * @code
-ksSort(ks);
- * @endcode
- *
- * somewhere afterwards before using ksLookup(). Otherwise the
- * renamed key won't be found.
- *
- * kdbSet() will have no problems with that issue, because it
- * duplicates its keysets using ksDup().
- *
- * @param ks the keyset object to work with
- * @see ksAppendKey(), ksAppend() and ksSort()
- * @return 1 if sort is needed
- *  0 if the keyset is sorted
  */
 int ksNeedSort (const KeySet *ks)
 {
-	return (ks->flags & KS_FLAG_DIRTY) == KS_FLAG_DIRTY;
+	return 0;
 }
 
 
 /* Used as a callback by the qsort() function */
-static int keyCompareWithRemove(const void *p1, const void *p2) {
+static int keyCmpInternal(const void *p1, const void *p2) {
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
 	const char *name2 = keyName(key2);
 	int ret = strcmp(name1, name2);
 
-	/* one key remove, sort in order of KEY_FLAG_REMOVE */
-	if (keyNeedRemove(key1) && !keyNeedRemove(key2)) return -1;
-	else if (!keyNeedRemove(key1) && keyNeedRemove(key2)) return 1;
-
-
-	if (keyNeedRemove(key1) && keyNeedRemove(key2))
-	{	/* both keys remove, sort reverse */
-		if (ret < 0) return 1;
-		else if (ret > 0) return -1;
-		else return 0;
-	} else { /* sort by owner */
-		if (ret == 0)
-		{
-			const char *owner1 = keyOwner(key1);
-			const char *owner2 = keyOwner(key2);
-			return strcmp(owner1, owner2);
-		}
-		else return ret;
+	/* sort by owner */
+	if (ret == 0)
+	{
+		const char *owner1 = keyOwner(key1);
+		const char *owner2 = keyOwner(key2);
+		if (!owner1 && !owner2) return 0;
+		if (!owner1) return -1;
+		if (!owner2) return 1;
+		return strcmp(owner1, owner2);
 	}
+
+	return ret;
 }
 
 
-/**
- * Sorts a KeySet alphabetically by Key name.
+/**Compare two keys.
  *
- * You need ksSort() only in few cases directly.
+ * @return a number less than, equal to or greater than zero if
+ *    k1 is found, respectively, to be less than, to match, or
+ *    be greater than k2.
  *
- * @section sortlookup Don't need to sort before using ksLookup
+ * The comparison is based on a strcmp of the keynames, and iff
+ * they match a strcmp of the owner will be used to distuingish.
+ * If even this matches the keys are found to be exactly the
+ * same and 0 is returned. These two keys can't be used in the same
+ * KeySet.
+ *
+ * keyCmp() defines the sorting order for a KeySet.
+ *
+ * The following 3 points are the rules for null values.
+ * They only take account when none of the preceding rules
+ * matched.
+ *
+ * - A null pointer will be found to be smaller than every other
+ * key. If both are null pointers, 0 is returned.
+ *
+ * - A null name will be found to be smaller than every other
+ * name. If both are null names, 0 is returned.
+ *
+ * - No owner will be found to be smaller then every other owner.
+ * If both don't have a owner, 0 is returned.
+ *
+ * @note the owner will only be used if the names are equal.
+ *
+ * Often is enough to know if the other key is
+ * less then or greater then the other one.
+ * But Sometimes you need more precise information,
+ * see keyRel().
+ *
+ * Given any Keys k1 and k2 constructed with keyNew(), following
+ * equation hold true:
  *
- * You don't need it if you just just kdbGet() and subsequent
- * ksLookup().
  * @code
-KeySet *ks = ksNew(0);
-kdbGet(h, ks, k, 0);
-// ksPop(), ksAppend() and ksAppendKey() allowed here
-ksLookup(ks, s, 0); // you dont need to sort ks
+// keyCmp(0,0) == 0
+// keyCmp(k1,0) ==  1
+// keyCmp(0,k2) == -1
  * @endcode
  *
- * This is because the KeySet tracks if it needs to be sorted
- * and ksLookup() will sort when needed.
+ * You can write similar equation for the other rules.
  *
- * @section sortiterate Sort when iterating
- *
- * Before you iterate over a keyset you have to sort it, if
- * you need it in a sorted way.
- *
- * To achieve that you can pass option_t::KDB_O_SORT to kdbGet()
- * and kdbSet(). Then you will receive a already sorted keyset
- * over which you can iterate.
+ * Here are some more examples with equation:
  * @code
-KeySet *ks = ksNew(0);
-kdbGet(h, ks, k, KDB_O_SORT);
-// no changes to keyset allowed
-ksRewind(ks);
-// now you can iterate over a sorted keyset
+Key *k1 = keyNew("user/a", KEY_END);
+Key *k2 = keyNew("user/b", KEY_END);
+
+// keyCmp(k1,k2) < 0
+// keyCmp(k2,k1) > 0
  * @endcode
  *
- * Its of course also possible to use ksLookup() once, because it
- * will sort the keyset too.
+ * @code
+Key *k1 = keyNew("user/a", KEY_OWNER, "markus", KEY_END);
+Key *k2 = keyNew("user/a", KEY_OWNER, "max", KEY_END);
+
+// keyCmp(k1,k2) < 0
+// keyCmp(k2,k1) > 0
+ * @endcode
  *
- * @section sortkey Sort when changing key
+ * @warning dont try to strcmp the keyName() yourself because
+ *      the used strcmp implementation is allowed to differ from
+ *      simple ascii comparison.
  *
- * @warning You should not use keySetName() or keyRemove() when a
- * key belongs to a keyset. When you are doing this, you always need to @p manually
- * sort @p all keysets where the key was before using ksLookup() (otherwise ksLookup()
- * won't find that keys), kdbGet() or kdbSet() methods.
+ * @param k1 the first key object to compare with
+ * @param k2 the second key object to compare with
  *
- * When you change a key's name or its remove status the order
- * which was previously correctly is then wrong. The keyset
- * does not recognize this, so the programmer has to take care
- * that ksSort() is called before any operation which needs
- * a sorted keyset (which are all ksLookup(), all kdbGet()
- * and all kdbSet() functions).
- *
- * @note You can remember that easily that all functions which get options
- * require one of the following:
- * - that you did not manipulate a keys name or a remove status
- * - that you pass KDB_O_SORT when you know that you manipulated at least one key
- * - that you ksSort() yourself after manipulating keys
- *
- * @section dirty Dirty KeySet
- *
- * When you use ksAppend(), ksAppendKey(), ksPop() the keyset is dirty
- * afterwards, which means that it needs to be sorted. This is done
- * automatically using a ksLookup() method and in ksGet() or ksSet()
- * (All methods which accept options).
- *
- * It won't be done if you just iterate over the keyset, so you might
- * use a ksLookup() or ksSort() first. ksLookup() will be more efficient
- * in that case, because it will only sort when needed. Don't pass
- * KDB_O_NOALL (it will deactivate the sorting feature),
- * see ksLookup() for more information.
- *
- * @param ks KeySet to be sorted
- * @see kdbGet(), kdbSet(), ksLookup() for some functions which may
- *     need sorting before they are called. (All functions which take
- *     options as arguments)
- * @see keySetName(), keySetBaseName(), keyAddBaseName() and keyRemove()
- *     for all methods which change the sorting state where the keyset
- *     can't track the change.
- * @see ksAppend(), ksAppendKey(), ksPop() for all methods which make
- *     a keyset dirty.
+ * @see ksAppendKey(), ksAppend() will compare keys when appending
+ * @see ksLookup() will compare keys during searching
+ * @ingroup keytest
  */
-void ksSort(KeySet *ks)
+int keyCmp (const Key *k1, const Key *k2)
 {
-	if (!ks) return;
+	if (!k1 && !k2) return 0;
+	if (!k1) return -1;
+	if (!k2) return 1;
 
-	ks->flags = ~KS_FLAG_DIRTY & ks->flags;
-	if (! ks->size) return;
+	if (!k1->key && !k2->key) return 0;
+	if (!k1->key) return -1;
+	if (!k2->key) return 1;
 
-	qsort(ks->array,ks->size,sizeof(Key *),keyCompareWithRemove);
+	return keyCmpInternal(&k1, &k2);
+}
+
+/**Checks if KeySet needs sync.
+ *
+ * When keys are changed this is reflected into keyNeedSync().
+ *
+ * But when keys are popped from a keyset this can't be seen
+ * by looking at the individual keys.
+ *
+ * ksNeedSync() allows the backends to know if a key was
+ * popped from the keyset to know that this keyset needs
+ * to be written out.
+ *
+ * @param ks the keyset to work with
+ * @return -1 on null keyset
+ * @return 0 if it does not need sync
+ * @return 1 if it needs sync
+ */
+int ksNeedSync(const KeySet *ks)
+{
+	if (!ks) return -1;
+
+	return (ks->flags & KS_FLAG_SYNC) == KS_FLAG_SYNC;
 }
 
 
@@ -542,6 +527,81 @@ ssize_t ksGetSize(const KeySet *ks)
  *           Filling up KeySets            *
  *******************************************/
 
+
+/**
+ * Binary search in a keyset.
+ *
+ * @code
+
+ssize_t result = ksSearchInternal(ks, toAppend);
+
+if (result >= 0)
+{
+	ssize_t position = result;
+	// Seems like the key already exist.
+} else {
+	ssize_t insertpos = -result-1;
+	// Seems like the key does not exist.
+}
+ * @endcode
+ *
+ * @param ks the keyset to work with
+ * @param toAppend the key to check
+ * @return position where the key is (>=0) if the key was found
+ * @return -insertpos -1 (< 0) if the key was not found 
+ *    so to get the insertpos simple do: -insertpos -1
+ */
+ssize_t ksSearchInternal(const KeySet *ks, const Key *toAppend)
+{
+	ssize_t left = 0;
+	ssize_t right = ks->size-1;
+	register int cmpresult = 1;
+	ssize_t middle = -1;
+	ssize_t insertpos = 0;
+#if DEBUG && VERBOSE
+	int c=0;
+#endif
+
+
+	while(1)
+	{
+#if DEBUG && VERBOSE
+		++c;
+#endif
+		if (right < left)
+		{
+			/* Nothing was found */
+			break;
+		}
+		middle = left + ((right-left)/2);
+		cmpresult = keyCmpInternal(&toAppend, &ks->array[middle]);
+		if (cmpresult > 0)
+		{
+			insertpos = left = middle + 1;
+		} else if (cmpresult == 0)
+		{
+			/* We have found it */
+			break;
+		} else {
+			insertpos = middle;
+			right = middle - 1;
+		}
+#if DEBUG && VERBOSE
+		/* This is even for verbose too verbose!
+		printf ("bsearch -- c: %d res: %d left: %zd middle: %zd right: %zd insertpos: %zd\n",
+			c, cmpresult, left, middle, right, insertpos);
+		*/
+#endif
+	}
+
+	if (!cmpresult)
+	{
+		return middle;
+	} else {
+		return -insertpos - 1;
+	}
+}
+
 /**
  * Appends a Key to the end of @p ks.
  *
@@ -552,12 +612,15 @@ ssize_t ksGetSize(const KeySet *ks)
  * The reference counter of the key will be incremented, and
  * thus toAppend is not const.
  *
- * The KeySet internal cursor is not moved.
+ * If the keyname already existed, it will be replaced with
+ * the new key.
+ *
+ * The KeySet internal cursor will be set to the new key.
  *
- * Makes the keyset dirty, see ksSort().
  *
  * @return the size of the KeySet after insertion
  * @return -1 on NULL pointers
+ * @return -1 if insertion failed, the key will be deleted then.
  * @param ks KeySet that will receive the key
  * @param toAppend Key that will be appended to ks
  * @see ksInsert(), ksInsertKeys(), ksAppend(), keyNew(), ksDel()
@@ -566,16 +629,62 @@ ssize_t ksGetSize(const KeySet *ks)
  */
 ssize_t ksAppendKey(KeySet *ks, Key *toAppend)
 {
+	ssize_t result = -1;
+
 	if (!ks) return -1;
 	if (!toAppend) return -1;
+	if (!toAppend->key)
+	{
+		keyDel (toAppend);
+		return -1;
+	}
+
+	result = ksSearchInternal(ks, toAppend);
+
+	if (result >= 0)
+	{
+		/* Seems like the key already exist. */
+		if (toAppend == ks->array[result])
+		{
+			/* user tried to insert the same key again */
+			return ks->size;
+		}
+
+		/* Pop the key in the result */
+		keyDecRef (ks->array[result]);
+		keyDel (ks->array[result]);
+
+		/* And use the other one instead */
+		keyIncRef (toAppend);
+		ks->array[result] = toAppend;
+		ksSetCursor(ks, result);
+	} else {
+		ssize_t insertpos = -result-1;
+
+		/* We want to append a new key
+		  in position insertpos */
+		++ ks->size;
+		if (ks->size >= ks->alloc) ksResize (ks, ks->alloc * 2-1);
+		keyIncRef (toAppend);
+
+		if (insertpos == (ssize_t)ks->size-1)
+		{
+			/* Append it to the very end */
+			ks->array[ks->size-1] = toAppend;
+			ks->array[ks->size] = 0;
+			ksSetCursor(ks, ks->size-1);
+		} else {
+			size_t n = ks->size-insertpos;
+			memmove(ks->array+(insertpos+1), ks->array+insertpos, n*sizeof(struct Key*));
+			/*
+			printf ("memmove -- ks->size: %zd insertpos: %zd n: %zd\n",
+				ks->size, insertpos, n);
+			*/
+			ks->array[insertpos] = toAppend;
+			ksSetCursor(ks, insertpos);
+		}
+	}
 
-	ks->flags |= KS_FLAG_DIRTY;
-	++ ks->size;
-	if (keyNeedRemove (toAppend)) ++ ks->rsize;
-	if (ks->size >= ks->alloc) ksResize (ks, ks->alloc * 2);
-	keyIncRef (toAppend);
-	ks->array[ks->size-1] = toAppend;
-	ks->array[ks->size] = 0;
 	return ks->size;
 }
 
@@ -586,8 +695,11 @@ ssize_t ksAppendKey(KeySet *ks, Key *toAppend)
  *
  * @p toAppend KeySet will be left unchanged.
  *
- * Makes the keyset dirty, see ksSort().
+ * If a key is both in toAppend and ks, the Key in ks will be
+ * overridden.
  *
+ * @post Sorted KeySet ks with all keys it had before and additionally
+ *       the keys from toAppend
  * @return the size of the KeySet after transfer
  * @return -1 on NULL pointers
  * @param ks the KeySet that will receive the keys
@@ -597,29 +709,187 @@ ssize_t ksAppendKey(KeySet *ks, Key *toAppend)
  */
 ssize_t ksAppend(KeySet *ks, const KeySet *toAppend)
 {
-	size_t oldSize = 0;
-	int i = 0;
-	int toAlloc = 0;
+	size_t toAlloc = 0;
 
 	if (!ks) return -1;
 	if (!toAppend) return -1;
 
-	oldSize = ks->size;
-	toAlloc = ks->alloc;
-
 	if (toAppend->size <= 0) return ks->size;
-	ks->flags |= KS_FLAG_DIRTY;
-	ks->size += toAppend->size;
-	ks->rsize += toAppend->rsize;
-	while (ks->size >= toAlloc) toAlloc *= 2;
-	ksResize (ks, toAlloc);
-	for (i=0; i<toAppend->size; i++) keyIncRef(toAppend->array[i]);
-	memcpy (ks->array + oldSize, toAppend->array, toAppend->size * sizeof (Key *));
-	ks->array[ks->size] = 0;
+
+	/* Do only one resize in advance */
+	for (toAlloc = ks->alloc; ks->size+toAppend->size >= toAlloc; toAlloc *= 2);
+	ksResize (ks, toAlloc-1);
+
+	/* TODO: here is lots of room for optimizations */
+	for (size_t i=0; i<toAppend->size; ++i)
+	{
+		ksAppendKey (ks, toAppend->array[i]);
+	}
 	return ks->size;
 }
 
 
+static int keyCompareByNameOwner(const void *p1, const void *p2);
+
+
+/*
+ * Returns the previous Key in a KeySet.
+ *
+ * KeySets have an internal cursor that can be reset with ksRewind(). Every
+ * time ksPrev() is called the cursor is decremented and the new current Key
+ * is returned.
+ *
+ * You'll get a NULL pointer if the key before begin of the KeySet was reached.
+ *
+ * Don't delete the key, use ksPop() if you want to delete it.
+ *
+ * @return the new current Key
+ * @see ksRewind(), ksCurrent()
+ *
+ */
+static Key *ksPrev(KeySet *ks)
+{
+	if (ks->size == 0) return 0;
+	if (ks->current <= 0)
+	{
+		ksRewind (ks);
+		return 0;
+	}
+	ks->current--;
+	return ks->cursor = ks->array[ks->current];
+}
+
+
+
+/**
+ * Copies all Keys until the end of the array from a position
+ * in the array to an position in the array.
+ *
+ * @param ks the keyset where this should be done
+ * @param to the position where it should be copied to
+ * @param from the position where it should be copied from
+ * @retval -1 if length is smaller then 0
+ * @return the number of moved elements otherwise
+ */
+ssize_t ksCopyInternal(KeySet *ks, size_t to, size_t from)
+{
+	ssize_t ssize = ks->size;
+	ssize_t sto = to;
+	ssize_t sfrom = from;
+
+	ssize_t sizediff = sto-sfrom;
+	ssize_t length = ssize-sfrom;
+	size_t ret = 0;
+
+	if (length < 0) return -1;
+	if (ks->size < to) return -1;
+
+	ks->size = ks->size + sizediff;
+	ret = elektraMemmove(ks->array + to, ks->array + from, length);
+	ks->array[ks->size] = 0;
+	return ret;
+}
+
+/**
+ * Cuts out a keyset at the cutpoint.
+ *
+ * Searches for the cutpoint inside the KeySet ks.
+ * If found it cuts out everything which is below (see keyIsBelow()) this key.
+ * If not found an empty keyset is returned.
+ *
+ * The cursor will stay at the same key as it was before.
+ * If the cursor was inside the region of cutted (moved)
+ * keys, the cursor will be set to the key before
+ * the cutpoint.
+ *
+ * @return a new allocated KeySet which needs to deleted with ksDel().
+ *         The keyset consists of all keys (of the original keyset ks)
+ *         below the cutpoint. If the key cutpoint exists, it will
+ *         also be appended.
+ * @retval 0 on null pointers, no key name or allocation problems
+ * @param ks the keyset to cut. It will be modified by removing
+ *           all keys below the cutpoint.
+ *           The cutpoint itself will also be removed.
+ * @param cutpoint the point where to cut out the keyset
+ */
+KeySet *ksCut(KeySet *ks, const Key *cutpoint)
+{
+	KeySet *returned = 0;
+	size_t found = 0;
+	size_t it = 0;
+	size_t newsize = 0;
+	int set_cursor = 0;
+
+	if (!ks) return 0;
+	if (!cutpoint) return 0;
+	if (!cutpoint->key) return 0;
+
+	// search the cutpoint
+	while (it < ks->size && keyIsBelowOrSame(cutpoint, ks->array[it]) == 0)
+	{
+		++it;
+	}
+
+	// we found nothing
+	if (it == ks->size) return ksNew (0);
+
+	// we found the cutpoint
+	found = it;
+
+	// search the end of the keyset to cut
+	while (it < ks->size && keyIsBelowOrSame(cutpoint, ks->array[it]) == 1)
+	{
+		++it;
+	}
+
+	// correct cursor if cursor is in cutted keyset
+	if (ks->current >= found && ks->current < it)
+	{
+		if (found == 0)
+		{
+			ksRewind(ks);
+		}
+		else
+		{
+			ks->current = found - 1;
+			set_cursor = 1;
+		}
+	}
+
+	// correct the cursor for the keys after the cutted keyset
+	if (ks->current >= it)
+	{
+		if (it >= ks->size)
+		{
+			ksRewind(ks);
+		}
+		else
+		{
+			ks->current = found + ks->current - it;
+			set_cursor = 1;
+		}
+	}
+
+	newsize = it-found;
+
+	returned = ksNew(newsize, KS_END);
+	elektraMemcpy (returned->array, ks->array+found, newsize);
+	returned->size = newsize;
+	returned->array[returned->size] = 0;
+
+	if (ksCopyInternal(ks, found, it) == -1)
+	{
+#if DEBUG
+		printf ("ksCopyInternal returned an error inside ksCut\n");
+#endif
+	}
+
+	if (set_cursor) ks->cursor = ks->array[ks->current];
+
+	return returned;
+}
+
+
 
 
 /**
@@ -639,7 +909,7 @@ ssize_t ksAppend(KeySet *ks, const KeySet *toAppend)
 ks1=ksNew(0);
 ks2=ksNew(0);
 
-k1=keyNew(0); // ref counter 0
+k1=keyNew("user/name", KEY_END); // ref counter 0
 ksAppendKey(ks1, k1); // ref counter 1
 ksAppendKey(ks2, k1); // ref counter 2
 
@@ -665,9 +935,12 @@ Key *ksPop(KeySet *ks)
 
 	if (!ks) return 0;
 
+	ks->flags |= KS_FLAG_SYNC;
+
 	if (ks->size <= 0) return 0;
+
 	-- ks->size;
-	if (ks->size+1 < ks->alloc/2) ksResize (ks, ks->alloc / 2);
+	if (ks->size+1 < ks->alloc/2) ksResize (ks, ks->alloc / 2-1);
 	ret = ks->array[ks->size];
 	ks->array[ks->size] = 0;
 	keyDecRef(ret);
@@ -692,7 +965,7 @@ Key *ksPop(KeySet *ks)
  *
  * @code
 ksRewind (ks);
-while ((key = keyNext (ks))!=0) {}
+while ((key = ksNext (ks))!=0) {}
  * @endcode
  *
  * @param ks the keyset object to work with
@@ -718,8 +991,7 @@ int ksRewind(KeySet *ks)
  * is returned.
  *
  * You'll get a NULL pointer if the key after the end of the KeySet was reached.
- * It will set the cursor to the beginning of
- * the KeySet and the next time the first key is returned.
+ * On subsequent calls of ksNext() it will still return the NULL pointer.
  *
  * The @p ks internal cursor will be changed, so it is not const.
  *
@@ -841,13 +1113,13 @@ Key *ksTail(const KeySet *ks)
  * @code
 cursor_t jump;
 ksRewind (ks);
-while ((key = keyNext (ks))!=0)
+while ((key = keyNextMeta (ks))!=0)
 {
 	// now mark this key
 	jump = ksGetCursor(ks);
 
 	//code..
-	keyNext (ks); // now browse on
+	keyNextMeta (ks); // now browse on
 	// use ksCurrent(ks) to check the keys
 	//code..
 
@@ -917,7 +1189,7 @@ cursor_t cursor;
 ..
 // key now in any position here
 cursor = ksGetCursor (ks);
-while ((key = keyNext (ks))!=0) {}
+while ((key = keyNextMeta (ks))!=0) {}
 ksSetCursor (ks, cursor); // reset state
 ksCurrent(ks); // in same position as before
  * @endcode
@@ -950,36 +1222,6 @@ int ksSetCursor(KeySet *ks, cursor_t cursor)
 
 
 
-/*
- * Returns the previous Key in a KeySet.
- *
- * KeySets have an internal cursor that can be reset with ksRewind(). Every
- * time ksPrev() is called the cursor is decremented and the new current Key
- * is returned.
- *
- * You'll get a NULL pointer if the key before begin of the KeySet was reached.
- *
- * Don't delete the key, use ksPop() if you want to delete it.
- *
- * @return the new current Key
- * @see ksRewind(), ksCurrent()
- *
- */
-static Key *ksPrev(KeySet *ks)
-{
-	if (ks->size == 0) return 0;
-	if (ks->current <= 0)
-	{
-		ksRewind (ks);
-		return 0;
-	}
-	ks->current--;
-	return ks->cursor = ks->array[ks->current];
-}
-
-
-
-
 /******************************************* 
  *    Looking up Keys inside KeySets       *
  *******************************************/
@@ -999,7 +1241,7 @@ static int keyCompareByNameCase(const void *p1, const void *p2) {
 	const char *name1 = keyName(key1);
 	const char *name2 = keyName(key2);
 
-	return kdbiStrCaseCmp(name1, name2);
+	return elektraStrCaseCmp(name1, name2);
 }
 
 static int keyCompareByNameOwner(const void *p1, const void *p2) {
@@ -1024,13 +1266,13 @@ static int keyCompareByNameOwnerCase(const void *p1, const void *p2) {
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
 	const char *name2 = keyName(key2);
-	int result = kdbiStrCaseCmp(name1, name2);
+	int result = elektraStrCaseCmp(name1, name2);
 
 	if (result == 0)
 	{
 		const char *owner1 = keyOwner(key1);
 		const char *owner2 = keyOwner(key2);
-		return kdbiStrCaseCmp(owner1, owner2);
+		return elektraStrCaseCmp(owner1, owner2);
 	}
 	else return result;
 }
@@ -1085,30 +1327,18 @@ static int keyCompareByNameOwnerCase(const void *p1, const void *p2) {
  * When KDB_O_NOALL is not set the cursor will stay untouched and all keys
  * are considered. A much more efficient binary search will be used then.
  *
- * So if you change keys, e.g. rename (keySetName()) or remove (keyRemove()) them
- * make sure to sort the keyset with ksSort(). When the keyset is dirty,
- * see ksNeedSort() it will be sorted automatically when needed.
- *
  * @subsection KDB_O_POP
  *
  * When KDB_O_POP is set the key which was found will be ksPop()ed. ksCurrent()
  * will not be changed, only iff ksCurrent() is the searched key, then the keyset
  * will be ksRewind()ed.
  *
- * @note Note that keyRemove() keys won't be found after the first time the keyset
- * is resorted. Lookup automatically sorts the keyset, if needed, but it
- * can't find it out when only keys are changed, not the keyset.
- *
  * @note Like in ksPop() the popped key always needs to be keyDel() afterwards, even
  * if it is appended to another keyset.
  *
  * @warning All cursors on the keyset will be invalid
  * iff you use KDB_O_POP, so don't use this if you rely on a cursor, see ksGetCursor().
  *
- * @note Never use ksLookup() with KDB_O_POP and ksAppendKey() or ksAppend() together in a loop.
- * Otherwise ksLookup() will need to resort the keyset every iteration and spend 99.96% of the
- * time in ksSort() (benchmarked with above 500k iterations).
- *
  * You can solve this problem by using KDB_O_NOALL, risking you have to iterate n^2 instead of n.
  *
  * The more elegant way is to separate the keyset you use for ksLookup() and ksAppendKey():
@@ -1144,14 +1374,12 @@ int f(KeySet *iterator, KeySet *lookup)
  * 		Only search from ksCurrent() to end of keyset, see above text.
  * 	- @p KDB_O_POP @n
  * 		Pop the key which was found.
- * 	- @p KDB_O_SORT @n
- * 		Force sorting before searching, see ksSort().
- * 		Together with KDB_O_NOALL the search will start from beginning.
+ *	- @p KDB_O_DEL @n
+ *		Delete the passed key.
  * @return pointer to the Key found, 0 otherwise
  * @return 0 on NULL pointers
  * @see ksLookupByName() to search by a name given by a string
  * @see ksCurrent(), ksRewind(), ksNext() for iterating over a keyset
- * @see ksSort() to understand how keyset sort themself
  */
 Key *ksLookup(KeySet *ks, Key * key, option_t options)
 {
@@ -1159,19 +1387,12 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
 	Key ** found;
 	cursor_t cursor = 0;
 	size_t jump = 0;
+	/*If there is a known offset in the beginning jump could be set*/
 
 	if (!ks) return 0;
 
-	jump = ks->rsize;
 	cursor = ksGetCursor (ks);
 
-	if (options & KDB_O_SORT)
-	{
-		ksSort (ks);
-		ksRewind (ks);
-	}
-	else if (!(options & KDB_O_NOALL) && ks->flags & KS_FLAG_DIRTY) ksSort(ks);
-
 	if (!key) return 0;
 
 	if (options & KDB_O_NOALL)
@@ -1224,6 +1445,10 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
 				{
 					ksRewind(ks);
 				}
+
+				ks->flags |= KS_FLAG_SYNC;
+
+				ksRewind(ks);
 				return ksPop(ks);
 			} else {
 				cursor = found-ks->array;
@@ -1255,6 +1480,7 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
  * (also accessible by ksCurrent()), and a pointer to the Key is returned.
  * If not found, @p ks internal cursor will not move, and a NULL pointer is
  * returned.
+ * If requested to pop the key, the cursor will be rewinded.
  *
  * @section cascading Cascading
  *
@@ -1293,9 +1519,6 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
  * 		Only search from ksCurrent() to end of keyset, see above text.
  * 	- @p KDB_O_POP @n
  * 		Pop the key which was found.
- * 	- @p KDB_O_SORT @n
- * 		Force sorting before searching, see ksSort().
- * 		Together with KDB_O_NOALL the search will start from beginning.
  *
  * 	Currently no options supported.
  * @return pointer to the Key found, 0 otherwise
@@ -1317,7 +1540,7 @@ Key *ksLookupByName(KeySet *ks, const char *name, option_t options)
 	if (name[0] == '/')
 	{
 		/* Will be freed by second key */
-		newname = kdbiMalloc (strlen (name) + sizeof ("system") + 1);
+		newname = elektraMalloc (strlen (name) + sizeof ("system") + 1);
 		if (!newname)
 		{
 			/*errno = KDB_ERR_NOMEM;*/
@@ -1337,7 +1560,7 @@ Key *ksLookupByName(KeySet *ks, const char *name, option_t options)
 			keyDel (key);
 		}
 
-		kdbiFree (newname);
+		elektraFree (newname);
 		return found;
 	} else {
 		key = keyNew (name, KEY_END);
@@ -1374,9 +1597,6 @@ while (key=ksLookupByString(ks,"my value",0)) {
  * @param options some @p KDB_O_* option bits. Currently supported:
  * 	- @p KDB_O_NOALL @n
  * 		Only search from ksCurrent() to end of keyset, see ksLookup().
- * 	- @p KDB_O_SORT @n
- * 		Force sorting before searching, see ksSort().
- * 		Together with KDB_O_NOALL the search will start from beginning.
  * 	- @p KDB_O_NOCASE @n
  * 	  Lookup ignoring case.
  * @return the Key found, 0 otherwise
@@ -1391,13 +1611,6 @@ Key *ksLookupByString(KeySet *ks, const char *value, option_t options)
 
 	if (!ks) return 0;
 
-	if (options & KDB_O_SORT)
-	{
-		ksSort (ks);
-		ksRewind (ks);
-	}
-	else if (!(options & KDB_O_NOALL) && ks->flags & KS_FLAG_DIRTY) ksSort(ks);
-
 	if (!(options & KDB_O_NOALL))
 	{
 		ksRewind (ks);
@@ -1413,7 +1626,7 @@ Key *ksLookupByString(KeySet *ks, const char *value, option_t options)
 		/*fprintf (stderr, "Compare %s with %s\n", keyValue(current), value);*/
 
 		if ((options & KDB_O_NOCASE) && 
-			!kdbiStrCaseCmp(keyValue(current),value)) break;
+			!elektraStrCaseCmp(keyValue(current),value)) break;
 		else if (!strcmp(keyValue(current),value)) break;
 	}
 
@@ -1445,9 +1658,6 @@ Key *ksLookupByString(KeySet *ks, const char *value, option_t options)
  * @param options some @p KDB_O_* option bits:
  * 	- @p KDB_O_NOALL @n
  * 		Only search from ksCurrent() to end of keyset, see above text.
- * 	- @p KDB_O_SORT @n
- * 		Force sorting before searching, see ksSort().
- * 		Together with KDB_O_NOALL the search will start from beginning.
  * @return the Key found, NULL otherwise
  * @return 0 on NULL pointer
  * @see ksLookupByString()
@@ -1462,13 +1672,6 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
 
 	if (!ks) return 0;
 
-	if (options & KDB_O_SORT)
-	{
-		ksSort (ks);
-		ksRewind (ks);
-	}
-	else if (!(options & KDB_O_NOALL) && ks->flags & KS_FLAG_DIRTY) ksSort(ks);
-
 	if (!(options & KDB_O_NOALL))
 	{
 		ksRewind (ks);
@@ -1483,12 +1686,12 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
 
 		if (!value)
 		{
-			if (!current->data) break;
+			if (!current->data.v) break;
 			else continue;
 		}
 
-		if (current->data && 
-			!memcmp(current->data,value,size)) break;
+		if (current->data.v && 
+			!memcmp(current->data.v,value,size)) break;
 	}
 
 	/* reached end of KeySet */
@@ -1536,7 +1739,7 @@ Key *ksLookupByBinary(KeySet *ks, const void *value, size_t size,
  *
  * No common parent is possible, so @p returnedCommonParent will contain nothing.
  *
- * This method will work correctly only on @link ksSort() sorted KeySets @endlink.
+ * This method will work correctly only sorted KeySets.
  *
  * @param working the Keyset to work with
  * @param returnedCommonParent a pre-allocated buffer that will receive the
@@ -1550,6 +1753,10 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
 	Key *current=0;
 	cursor_t init;
 	KeySet *ks;
+	ssize_t sMaxSize;
+
+	if (maxSize > SSIZE_MAX) return -1;
+	sMaxSize = maxSize;
 
 	init = ksGetCursor (working);
 	ks = (KeySet *) working;
@@ -1558,14 +1765,14 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
 
 	ksRewind(ks);
 	current = ksNext(ks);
-	if (keyGetNameSize(current) > maxSize) {
+	if (keyGetNameSize(current) > sMaxSize) {
 		/*errno=KDB_ERR_TRUNC;*/
 		returnedCommonParent[0]=0;
 		return -1;
 	}
 
 	strcpy(returnedCommonParent,keyName(current));
-	parentSize=kdbiStrLen(returnedCommonParent);
+	parentSize=elektraStrLen(returnedCommonParent);
 
 	while (*returnedCommonParent) {
 		ksRewind(ks);
@@ -1580,7 +1787,7 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
 
 			if ((delim=strrchr(returnedCommonParent,PATH_SEPARATOR))) {
 				*delim=0;
-				parentSize=kdbiStrLen(returnedCommonParent);
+				parentSize=elektraStrLen(returnedCommonParent);
 			} else {
 				*returnedCommonParent=0;
 				parentSize=0;
@@ -1625,10 +1832,13 @@ ssize_t ksGetCommonParentName(const KeySet *working,char *returnedCommonParent,
  * @return 1 on success
  * @return 0 on nothing done because keyset would be too small.
  * @return -1 if alloc is smaller then current size of keyset.
- * @return -1 on memory error
+ * @return -1 on memory error or null ptr
  */
 int ksResize (KeySet *ks, size_t alloc)
 {
+	if (!ks) return -1;
+
+	alloc ++; /* for ending null byte */
 	if (alloc == ks->alloc) return 1;
 	if (alloc < ks->size) return 0;
 	if (alloc < KEYSET_SIZE)
@@ -1641,26 +1851,27 @@ int ksResize (KeySet *ks, size_t alloc)
 	{	/* Not allocated up to now */
 		ks->alloc = alloc;
 		ks->size = 0;
-		ks->array = kdbiMalloc (sizeof(struct _Key *) * ks->alloc);
+		ks->array = elektraMalloc (sizeof(struct _Key *) * ks->alloc);
 		if (!ks->array)
 		{
 			/*errno = KDB_ERR_NOMEM;*/
 			return -1;
 		}
 	}
-
+/* This is much too verbose
 #if DEBUG && VERBOSE
 	printf ("Resize from %d to %d\n",(int) ks->alloc,(int) alloc);
 #endif
+*/
 	ks->alloc=alloc;
 
 
-	if (kdbiRealloc((void**) &ks->array, sizeof(struct _Key *) * ks->alloc) == -1)
+	if (elektraRealloc((void**) &ks->array, sizeof(struct _Key *) * ks->alloc) == -1)
 	{
 #if DEBUG
 		fprintf (stderr, "Reallocation error\n");
 #endif
-		kdbiFree (ks->array);
+		elektraFree (ks->array);
 		ks->array = 0;
 		/*errno = KDB_ERR_NOMEM;*/
 		return -1;
@@ -1672,11 +1883,14 @@ int ksResize (KeySet *ks, size_t alloc)
 /*
  * Returns current allocation size.
  *
+ * This is the maximum size before a reallocation
+ * happens.
+ *
  * @param ks the keyset object to work with
  * @return allocated size*/
 size_t ksGetAlloc (const KeySet *ks)
 {
-	return ks->alloc;
+	return ks->alloc-1;
 }
 
 
@@ -1696,11 +1910,10 @@ size_t ksGetAlloc (const KeySet *ks)
  */
 int ksInit(KeySet *ks) {
 	ks->array = 0;
-	ks->flags = KS_FLAG_DIRTY;
 
 	ks->size=0;
-	ks->rsize=0;
 	ks->alloc=0;
+	ks->flags=0;
 
 	ksRewind(ks);
 
@@ -1726,12 +1939,11 @@ int ksClose(KeySet *ks)
 		keyDel (k);
 	}
 
-	if (ks->array) kdbiFree (ks->array);
+	if (ks->array) elektraFree (ks->array);
 	ks->array = 0;
 	ks->alloc = 0;
 
 	ks->size = 0;
-	ks->rsize= 0;
 
 	return 0;
 }
