@@ -13,8 +13,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_KDBCONFIG_H
+#include "kdbconfig.h"
 #endif
 
 #include <tests.h>
@@ -49,7 +49,7 @@ struct test tstKeyName[] =
 	{ "Normal user key", "user/key",
 		"user/key", 			/* keyName 	*/
 		"key", 				/* keyBaseName 	*/
-		"user:env1",			/* keyGetFullRootName 	*/ 
+		"user",				/* keyGetFullRootName 	*/ 
 		"user"				/* keyGetParentName	*/
 	
 	},
@@ -107,9 +107,8 @@ void test_keyComparing()
 	succeed_if(keyCompare(key1,key2) == 0, "the keys should not differ in owner");
 
 	keySetString (key1, "myvalue");
-	succeed_if(keyCompare(key1,key2) == (KEY_VALUE|KEY_TYPE), "the keys should differ in value and type");
-	keySetType (key2, KEY_TYPE_STRING);
-	succeed_if(keyCompare(key1,key2) == KEY_VALUE, "the keys should differ in value");
+	succeed_if(keyCompare(key1,key2) == (KEY_VALUE), "the keys should differ in value");
+
 	keySetString (key2, "myvalue");
 	succeed_if(keyCompare(key1,key2) == 0, "the keys should not differ in value");
 
@@ -133,11 +132,6 @@ void test_keyComparing()
 	keySetMode (key2, 0222);
 	succeed_if(keyCompare(key1,key2) == 0, "the keys should not differ in mode");
 
-	keySetType (key1, 50);
-	succeed_if(keyCompare(key1,key2) == KEY_TYPE, "the keys should differ in type");
-	keySetType (key2, 50);
-	succeed_if(keyCompare(key1,key2) == 0, "the keys should not differ in type");
-
 	keyDel (key1);
 	keyDel (key2);
 }
@@ -161,25 +155,22 @@ void test_keyNewSystem()
 	// Key with name
 	key = keyNew("system/sw/test", KEY_END);
 	succeed_if(key != NULL, "keyNew: Unable to create a key with name");
-	succeed_if(keyNeedRemove (key) == 0, "KEY_REMOVE set");
 	succeed_if(strcmp(keyName(key), "system/sw/test") == 0, "keyNew: Key's name setted incorrectly");
 	keyCopy (key, 0);
 	succeed_if (strcmp (keyName(key), "") == 0, "name after keyCopy(,0)");
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name");
 	
 	// Key with name
-	key = keyNew("system/sw/test", KEY_REMOVE, KEY_END);
+	key = keyNew("system/sw/test", KEY_END);
 	succeed_if(key != NULL, "keyNew: Unable to create a key with name");
-	succeed_if(keyNeedRemove (key) == 1, "KEY_REMOVE not set");
 	succeed_if(strcmp(keyName(key), "system/sw/test") == 0, "keyNew: Key's name setted incorrectly");
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name");
 	
-	// Key with name + value (default type must be KEY_TYPE_STRING)
+	// Key with name + value
 	key = keyNew("system/sw/test",
 			KEY_VALUE, "test",
 			KEY_END);
 	succeed_if(key != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(key), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
 	succeed_if(strcmp(keyValue(key), "test") == 0, "keyNew: Value not set correctly");
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name + value");
 	
@@ -202,7 +193,7 @@ void test_keyNewSystem()
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name + mode");
 
 	key = keyNew("system/valid/there",
-			KEY_TYPE, KEY_TYPE_BINARY,
+			KEY_BINARY,
 			KEY_SIZE, sizeof(array),
 			KEY_VALUE, array,
 			KEY_END);
@@ -228,15 +219,15 @@ void test_keyNewSystem()
 	k2=keyNew("system/2",   KEY_VALUE, "myvalue", KEY_END);
 	k3=keyNew("system/3", KEY_VALUE, "syskey",  KEY_END);
 	succeed_if(k1 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k1), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k1), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k1), "singlevalue") == 0, "keyNew: Value not set correctly");
 	
 	succeed_if(k2 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k2), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k2), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k2), "myvalue") == 0, "keyNew: Value not set correctly");
 	
 	succeed_if(k3 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k3), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k3), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k3), "syskey") == 0, "keyNew: Value not set correctly");
 
 	succeed_if(keyDel(k1) == 0, "keyDel: Unable to delete key with name + value");
@@ -255,13 +246,9 @@ void test_keyNewUser()
 	Key *k3;
 
 	printf("Test user key creation\n");
-	
-#ifdef HAVE_SETENV
-	setenv ("USER","hugo",1);
-#else
-	putenv ("USER=hugo");
-#endif
+
 	key = keyNew ("user/test/test", KEY_END);
+	succeed_if (keySetOwner(key, "hugo") == sizeof("hugo"), "could not set owner");
 	succeed_if( strcmp(keyOwner(key), "hugo") == 0, "keyNew: owner not set correctly");
 	succeed_if( keyGetOwnerSize(key) == 5, "owner length not correct");
 	keyGetFullName (key, fullroot, MAX_PATH_LENGTH);
@@ -269,15 +256,12 @@ void test_keyNewUser()
 	/* printf ("%s, %s, %s\n", keyName(key), keyBaseName(key), fullroot); */
 	succeed_if(strcmp(keyName(key),"user/test/test") == 0, "Wrong keyname: keyName");
 	succeed_if(strcmp(keyBaseName(key),"test") == 0, "Wrong keyname: keyBaseName");
+	printf ("%s\n", fullroot);
 	succeed_if(strcmp(fullroot,"user:hugo/test/test") == 0, "Wrong keyname: keyGetFullName");
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name + owner");
 
-#ifdef HAVE_SETENV	
-	setenv ("KDB_USER","tommy",1);
-#else
-	putenv("KDB_USER=tommy");
-#endif
 	key = keyNew ("user/test/test", KEY_END);
+	succeed_if (keySetOwner(key, "tommy") == sizeof("tommy"), "could not set owner");
 	succeed_if( strcmp(keyOwner(key), "tommy") == 0, "keyNew: owner not set correctly");
 	succeed_if( keyGetOwnerSize(key) == 6, "owner length not correct");
 	keyGetFullName (key, fullroot, MAX_PATH_LENGTH);
@@ -297,7 +281,7 @@ void test_keyNewUser()
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name + owner");
 	
 	key = keyNew("user/valid/there",
-			KEY_TYPE, KEY_TYPE_BINARY,
+			KEY_BINARY,
 			KEY_SIZE, sizeof(array),
 			KEY_VALUE, array,
 			KEY_END);
@@ -329,15 +313,15 @@ void test_keyNewUser()
 	k2=keyNew("user/2",   KEY_VALUE, "myvalue", KEY_END);
 	k3=keyNew("user/3", KEY_VALUE, "syskey",  KEY_END);
 	succeed_if(k1 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k1), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k1), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k1), "singlevalue") == 0, "keyNew: Value not set correctly");
 	
 	succeed_if(k2 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k2), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k2), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k2), "myvalue") == 0, "keyNew: Value not set correctly");
 	
 	succeed_if(k3 != NULL, "keyNew: Unable to create a key with name + value of default type");
-	succeed_if(keyIsString(k3), "keyNew: Default key value isn't set to KEY_TYPE_STRING");
+	succeed_if(keyIsString(k3), "keyNew: Default key value isn't set to string");
 	succeed_if(strcmp(keyValue(k3), "syskey") == 0, "keyNew: Value not set correctly");
 
 	succeed_if(keyDel(k1) == 0, "keyDel: Unable to delete key with name + value");
@@ -355,8 +339,8 @@ void test_keyReference()
 {
 	printf("Test key reference\n");
 
-	Key *key = keyNew (KEY_END);
-	Key *c = keyNew (KEY_END);
+	Key *key = keyNew ("user/key", KEY_END);
+	Key *c = keyNew ("user/c", KEY_END);
 	Key *d;
 	KeySet *ks1, *ks2;
 	succeed_if (keyGetRef(key) == 0, "New created key reference");
@@ -421,7 +405,7 @@ void test_keyReference()
 
 	ks1 = ksNew(0);
 	ks2 = ksNew(0);
-	key = keyNew(0); // ref counter 0
+	key = keyNew("user/key", KEY_END); // ref counter 0
 	succeed_if (keyGetRef(key) == 0, "reference counter");
 	ksAppendKey(ks1, key); // ref counter of key 1
 	succeed_if (keyGetRef(key) == 1, "reference counter");
@@ -471,47 +455,13 @@ void test_keyReference()
 
 }
 
-void test_keyType()
-{
-	Key * key;
-	char array [] = "test";
-	int i;
-
-	printf("Test key type\n");
-
-	key = keyNew (KEY_END);
-
-	for (i=0; i<256; i++)
-	{
-		keySetType (key, i);
-		succeed_if (keyGetType (key) == (i), "could not change to dir type to something between 0 and 255");
-	}
-
-	keySetType (key, 0);
-	succeed_if (keyGetType (key) == 0, "could not unset type");
-
-	succeed_if (keySetString (key, array) == 5, "size not correct");
-	succeed_if (keyIsString (key), "could not change to string type");
-	succeed_if (keyGetValueSize (key) == sizeof (array), "size is not correct");
-
-	succeed_if (keySetBinary (key, array, 4) == 4, "size not correct");
-	succeed_if (keyIsBinary (key), "could not change to binary type");
-	succeed_if (keyGetValueSize (key) == sizeof (array)-1, "size is not correct");
-	
-	succeed_if (keySetString (key, array) == 5, "size not correct");
-	succeed_if (keyIsString (key), "could not change to string type");
-	succeed_if (keyGetValueSize (key) == sizeof (array), "size is not correct");
-
-	keyDel (key);
-}
-
 void test_keyName()
 {
 	Key	*key;
 	size_t	size;
 	char	*buf;
 	char	ret [1000];
-	int	i;
+	size_t	i;
 	char *getBack;
 	char testName[] = "user/name";
 	char testFullName[] = "user:max/name";
@@ -899,23 +849,10 @@ void test_keyName()
 	succeed_if(strcmp(keyName(key), "user/validname\\/") == 0, "keyNew: Key's name setted incorrectly");
 	succeed_if(keyDel(key) == 0, "keyDel: Unable to delete key with name");
 
-#ifdef HAVE_CLEARENV
-	clearenv();
-#else
-	unsetenv("USER");
-	unsetenv("KDB_USER");
-#endif
-
-#ifdef HAVE_SETENV
-	setenv ("USER", "env1", 1);
-#else
-	putenv ("USER=env1");
-#endif
-
 	printf("Test key's name manipulation\n");
 	for(i = 0 ; tstKeyName[i].testName != NULL ; i++) {
 		key = keyNew(tstKeyName[i].keyName, KEY_END);
-		
+
 		/* keyName */
 		succeed_if( (strcmp(keyName(key), tstKeyName[i].expectedKeyName) == 0) , "keyName" );
 
@@ -924,10 +861,10 @@ void test_keyName()
 
 		/* keyGetFullRootNameSize */
 		size = keyGetFullRootNameSize(key);
-		succeed_if( (size == kdbiStrLen(tstKeyName[i].expectedFRootName)), "keyGetFullRootNameSize" );
+		succeed_if( (size == elektraStrLen(tstKeyName[i].expectedFRootName)), "keyGetFullRootNameSize" );
 		
 		/* keyGetFullRootName */
-		buf = kdbiMalloc(size*sizeof(char));
+		buf = elektraMalloc(size*sizeof(char));
 		keyGetFullRootName(key, buf, size);
 		// printf ("comp: %s - %s\n", buf, tstKeyName[i].expectedFRootName);
 		succeed_if( (strncmp(buf, tstKeyName[i].expectedFRootName, size) == 0), "keyGetFullRootName" );
@@ -935,40 +872,40 @@ void test_keyName()
 
 		/* keyGetParentNameSize */
 		size = keyGetParentNameSize(key);
-		succeed_if( (size == kdbiStrLen(tstKeyName[i].expectedParentName)), "ketGetParentNameSize" );
+		succeed_if( (size == elektraStrLen(tstKeyName[i].expectedParentName)), "ketGetParentNameSize" );
 
 		/* keyGetParentName */
 		size = keyGetParentNameSize(key)+1;
-		buf = kdbiMalloc(size*sizeof(char));
+		buf = elektraMalloc(size*sizeof(char));
 		keyGetParentName(key, buf, size);
 		succeed_if( (strncmp(buf, tstKeyName[i].expectedParentName, size) == 0), "keyGetParentName" );
 		free(buf);
 
 		/* keyGetBaseNameSize */
 		size = keyGetBaseNameSize(key);
-		succeed_if( (size == kdbiStrLen(tstKeyName[i].expectedBaseName)), "keyGetBaseNameSize" );
+		succeed_if( (size == elektraStrLen(tstKeyName[i].expectedBaseName)), "keyGetBaseNameSize" );
 
 		/* keyGetBaseName */
 		size = keyGetBaseNameSize(key)+1;
-		buf = kdbiMalloc(size*sizeof(char));
+		buf = elektraMalloc(size*sizeof(char));
 		keyGetBaseName(key, buf, size);
 		succeed_if( (strncmp(buf, tstKeyName[i].expectedBaseName, size) == 0), "keyGetBaseName" );
 		free(buf);
 
 		/* keyGetNameSize */
 		size = keyGetNameSize(key);
-		succeed_if( (size == kdbiStrLen(tstKeyName[i].expectedKeyName)), "keyGetKeyNameSize" );
+		succeed_if( (size == elektraStrLen(tstKeyName[i].expectedKeyName)), "keyGetKeyNameSize" );
 		
 		/* keyGetName */
 		size = keyGetNameSize(key);
-		buf = kdbiMalloc(size*sizeof(char));
+		buf = elektraMalloc(size*sizeof(char));
 		keyGetName(key, buf, size);
 		succeed_if( (strcmp(buf, tstKeyName[i].expectedKeyName) == 0), "keyGetName" );
 		free(buf);
 
 		succeed_if (keyGetRef (copy) == 0, "reference of copy not correct");
 		keyCopy (copy, key);
-		succeed_if (compare_key (copy, key, 0) == 0, "keys are not equal after copy");
+		succeed_if (compare_key (copy, key) == 0, "keys are not equal after copy");
 		succeed_if (keyGetRef (copy) == 0, "reference of copy not correct");
 
 		keyDel(key);
@@ -981,10 +918,10 @@ void test_keyValue()
 {
 	Key * key;
 	char	ret [1000];
-	int i;
+	size_t i;
 	char testString[] = "teststring";
 	char testBinary[] = "\0tes\1tbinary";
-	testBinary[sizeof(testBinary)] = 'T';
+	testBinary[sizeof(testBinary)-1] = 'T';
 
 	printf("Test value of keys\n");
 
@@ -1017,17 +954,13 @@ void test_keyValue()
 	succeed_if (key = keyNew(0), "could not create new key");
 	succeed_if (keySetString (key, "a long long string") == 19, "could not set string");
 	succeed_if (keyGetString (key, ret, 6) == -1, "string not truncated");
-	/*succeed_if (errno == KDB_ERR_TRUNC, "errno not set properly trunc");*/
 	succeed_if (keyGetBinary (key, ret, 999) == -1, "binary not mismatch");
-	/*succeed_if (errno == KDB_ERR_TYPEMISMATCH, "errno not set properly typemismatch");*/
 	succeed_if (keyDel (key) == 0, "could not delete key");
 
 	succeed_if (key = keyNew(0), "could not create new key");
 	succeed_if (keySetBinary (key, "a long long binary", 19) == 19, "could not set string");
 	succeed_if (keyGetBinary (key, ret, 6) == -1, "binary not truncated");
-	/*succeed_if (errno == KDB_ERR_TRUNC, "errno not set properly trunc");*/
 	succeed_if (keyGetString (key, ret, 999) == -1, "string not mismatch");
-	/*succeed_if (errno == KDB_ERR_TYPEMISMATCH, "errno not set properly typemismatch");*/
 	succeed_if (keyDel (key) == 0, "could not delete key");
 
 	succeed_if (key = keyNew(0), "could not create new key");
@@ -1150,21 +1083,19 @@ void test_keyBinary(void)
 	Key *key =0;
 	char ret [1000];
 	int i;
-	char c;
 	char binaryData[] = "\0binary \1\34data";
-	binaryData[sizeof(binaryData)] = 'T';
+	binaryData[sizeof(binaryData)-1] = 'T';
 
 	printf ("Test binary special cases\n");
 
 	key = keyNew ("user/binary",
-		KEY_TYPE, KEY_TYPE_BINARY+1,
+		KEY_BINARY,
 		KEY_SIZE, sizeof(binaryData),
 		KEY_VALUE, binaryData,
 		KEY_END);
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == sizeof(binaryData), "size not correct");
 	succeed_if (memcmp(binaryData, keyValue(key), sizeof(binaryData)) == 0, "memcmp");
 	succeed_if (keyGetBinary(key, ret, 1000) == sizeof(binaryData), "could not get binary data");
@@ -1174,12 +1105,10 @@ void test_keyBinary(void)
 	keyDel (key);
 
 	key = keyNew(0);
-	keySetType (key, KEY_TYPE_BINARY+1);
 	keySetBinary (key, binaryData, sizeof(binaryData));
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == sizeof(binaryData), "size not correct");
 	succeed_if (memcmp(binaryData, keyValue(key), sizeof(binaryData)) == 0, "memcmp");
 	succeed_if (keyGetBinary(key, ret, 1000) == sizeof(binaryData), "could not get binary data");
@@ -1188,26 +1117,11 @@ void test_keyBinary(void)
 
 	keyDel (key);
 
-	key = keyNew(0);
-	keySetType (key, KEY_TYPE_BINARY+1);
-
-	succeed_if (keyIsBinary(key) == 1, "should be binary");
-	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
-	succeed_if (keyGetValueSize(key) == 0, "size not correct");
-	succeed_if (keyValue(key) == 0, "should be null pointer");
-	succeed_if (keyGetBinary(key, ret, 1000) == 0, "should write nothing because of no data");
-	succeed_if (keyGetString(key, ret, 1000) == -1, "should be type mismatch");
-
-	keyDel (key);
-
 	key = keyNew(0);
 	keySetBinary(key, 0, 0);
-	keySetType (key, KEY_TYPE_BINARY+1);
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == 0, "size not correct");
 	succeed_if (keyValue(key) == 0, "should be null pointer");
 	succeed_if (keyGetBinary(key, ret, 1000) == 0, "should write nothing because of no data");
@@ -1217,11 +1131,9 @@ void test_keyBinary(void)
 
 	key = keyNew(0);
 	keySetBinary(key, 0, 1);
-	keySetType (key, KEY_TYPE_BINARY+1);
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == 0, "size not correct");
 	succeed_if (keyValue(key) == 0, "should be null pointer");
 	succeed_if (keyGetBinary(key, ret, 1000) == 0, "should write nothing because of no data");
@@ -1233,11 +1145,9 @@ void test_keyBinary(void)
 	keySetBinary(key, "", 1);
 	succeed_if (keySetBinary(key, 0, SIZE_MAX) == -1, "should do nothing and fail");
 	succeed_if (keySetBinary(key, 0, SSIZE_MAX) == 0, "should free data");
-	keySetType (key, KEY_TYPE_BINARY+1);
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == 0, "size not correct");
 	succeed_if (keyValue(key) == 0, "should be null pointer");
 	succeed_if (keyGetBinary(key, ret, 1000) == 0, "should write nothing because of no data");
@@ -1247,11 +1157,9 @@ void test_keyBinary(void)
 
 	key = keyNew(0);
 	keySetBinary(key, "", 1);
-	keySetType (key, KEY_TYPE_BINARY+1);
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == 1, "size not correct");
 	succeed_if (memcmp(binaryData, keyValue(key), 1) == 0, "memcmp");
 	succeed_if (keyGetBinary(key, ret, 1000) == 1, "could not get binary data");
@@ -1262,12 +1170,10 @@ void test_keyBinary(void)
 
 	key = keyNew(0);
 	i = 23;
-	keySetType (key, KEY_TYPE_BINARY+1);
 	keySetBinary (key, (void*)&i, sizeof(i));
 
 	succeed_if (keyIsBinary(key) == 1, "should be binary");
 	succeed_if (keyIsString(key) == 0, "should not be string");
-	succeed_if (keyGetType(key) == KEY_TYPE_BINARY+1, "forgot its type");
 	succeed_if (keyGetValueSize(key) == sizeof(i), "size not correct");
 	succeed_if (memcmp((void*)&i, keyValue(key), sizeof(i)) == 0, "memcmp");
 	succeed_if (keyGetBinary(key, ret, 1000) == sizeof(i), "could not get binary data");
@@ -1285,7 +1191,7 @@ void test_keyComment()
 {
 	Key * key;
 	char	ret [1000];
-	int i;
+	size_t i;
 	char testComment [] = "testcomment";
 
 	printf("Test comment of key\n");
@@ -1308,7 +1214,7 @@ void test_keyComment()
 	succeed_if (key = keyNew(0), "could not create new key");
 	succeed_if (strcmp (keyComment(key), "") == 0, "Empty comment problem");
 	succeed_if (keyGetCommentSize(key) == 1, "Empty comment size problem");
-	succeed_if (keySetComment (key,"") == 0, "could not set comment");
+	succeed_if (keySetComment (key,"") == 1, "could not set comment");
 	succeed_if (strcmp (keyComment(key), "") == 0, "Empty comment problem");
 	succeed_if (keyGetCommentSize(key) == 1, "Empty comment size problem");
 	succeed_if (keyGetComment(key, ret, 0) == -1, "Could not get empty comment");
@@ -1569,7 +1475,7 @@ void test_keyDup()
 
 	// Create test key
 	orig = keyNew("user:yl/foo/bar",
-			KEY_TYPE, KEY_TYPE_BINARY,
+			KEY_BINARY,
 			KEY_SIZE, 6,
 			KEY_VALUE, "foobar",
 			KEY_COMMENT, "mycomment", 
@@ -1597,7 +1503,7 @@ void test_keyDup()
 	succeed_if( keyGetUID(copy) == 123, "keyDup: key UID copy error");
 	succeed_if( keyGetGID(copy) == 456, "keyDup: key GID copy error");
 	succeed_if( keyGetMode(copy) == 0644, "keyDup: key mode copy error");
-	succeed_if( keyGetType(copy) & (KEY_TYPE_BINARY), "keyDup: key type copy error");
+	succeed_if (keyIsBinary(copy), "keyDup: key type copy error");
 
 	keyDel(copy);
 
@@ -1613,8 +1519,6 @@ void test_keyDup()
 	succeed_if ( strcmp(keyOwner(copy), "") == 0, "copy name should be empty");
 	succeed_if (keyGetOwnerSize(orig) == 1, "orig name size");
 	succeed_if (keyGetOwnerSize(copy) == 1, "copy name size");
-	succeed_if (keyGetType(orig) == KEY_TYPE_UNDEFINED, "orig key type");
-	succeed_if (keyGetType(copy) == KEY_TYPE_UNDEFINED, "copy key type");
 	succeed_if (keyGetATime(orig) == keyGetATime(copy), "ATime should be same");
 	succeed_if (keyGetCTime(orig) == keyGetCTime(copy), "CTime should be same");
 	succeed_if (keyGetMTime(orig) == keyGetMTime(copy), "MTime should be same");
@@ -1633,7 +1537,7 @@ void test_keyCopy()
 
 	// Create test key
 	orig = keyNew("user:yl/foo/bar",
-			KEY_TYPE, KEY_TYPE_BINARY,
+			KEY_BINARY,
 			KEY_SIZE, 6,
 			KEY_VALUE, "foobar",
 			KEY_COMMENT, "mycomment", 
@@ -1662,12 +1566,11 @@ void test_keyCopy()
 	succeed_if( keyGetUID(copy) == 123, "keyCopy: key UID copy error");
 	succeed_if( keyGetGID(copy) == 456, "keyCopy: key GID copy error");
 	succeed_if( keyGetMode(copy) == 0644, "keyCopy: key mode copy error");
-	succeed_if( keyGetType(copy) & (KEY_TYPE_BINARY), "keyCopy: key type copy error");
 
 	orig = keyNew(0);
 	succeed_if (keyCopy(copy, 0) == 0, "make the key copy fresh");
 
-	succeed_if (compare_key (orig, copy, 0) == 0, "keyCopy did not make the key fresh");
+	succeed_if (compare_key (orig, copy) == 0, "keyCopy did not make the key fresh");
 	keyDel (orig);
 
 	keyDel(copy);
@@ -1685,8 +1588,6 @@ void test_keyCopy()
 	succeed_if ( strcmp(keyOwner(copy), "") == 0, "copy name should be empty");
 	succeed_if (keyGetOwnerSize(orig) == 1, "orig name size");
 	succeed_if (keyGetOwnerSize(copy) == 1, "copy name size");
-	succeed_if (keyGetType(orig) == KEY_TYPE_UNDEFINED, "orig key type");
-	succeed_if (keyGetType(copy) == KEY_TYPE_UNDEFINED, "copy key type");
 	succeed_if (keyGetATime(orig) == keyGetATime(copy), "ATime should be same");
 	succeed_if (keyGetCTime(orig) == keyGetCTime(copy), "CTime should be same");
 	succeed_if (keyGetMTime(orig) == keyGetMTime(copy), "MTime should be same");
@@ -1699,7 +1600,7 @@ void test_keyCopy()
 
 void test_keyDir (void)
 {
-	int i;
+	mode_t i;
 	Key * key = keyNew ("user", KEY_END);
 
 	printf ("Test directory keys\n");
@@ -1774,9 +1675,9 @@ void test_keyTime()
 
 	printf ("Test key time\n");
 
-	succeed_if (keyGetATime(0) == (size_t)-1, "null pointer check");
-	succeed_if (keyGetMTime(0) == (size_t)-1, "null pointer check");
-	succeed_if (keyGetCTime(0) == (size_t)-1, "null pointer check");
+	succeed_if (keyGetATime(0) == (time_t)-1, "null pointer check");
+	succeed_if (keyGetMTime(0) == (time_t)-1, "null pointer check");
+	succeed_if (keyGetCTime(0) == (time_t)-1, "null pointer check");
 
 	succeed_if (keySetATime(0,0) == -1, "null pointer check");
 	succeed_if (keySetMTime(0,0) == -1, "null pointer check");
@@ -1833,49 +1734,21 @@ void test_keyMeta(void)
 {
 	Key *key=0;
 
-	succeed_if (keyStat (key) == -1, "stat null pointer");
-	succeed_if (keyNeedStat(key) == -1, "could not stat null pointer");
 	succeed_if(keyGetUID(key) == (uid_t)-1, "uid null pointer");
 	succeed_if(keyGetGID(key) == (gid_t)-1, "gid null pointer");
 	succeed_if(keyGetMode(key) == (mode_t)-1, "mode null pointer");
 
-
-	key = keyNew (0);
-
-	succeed_if (keyNeedStat(key) == 0, "fresh key does not need stat");
-	succeed_if (keyStat(key) == 1, "could not stat key");
-	succeed_if (keyNeedStat(key) == 1, "could not stat key");
-	keyDel (key);
-
-	key = keyNew ("user/stat", KEY_STAT, KEY_END);
-	succeed_if (keyNeedStat(key) == 1, "fresh key need stat");
-	succeed_if (keyStat(key) == 1, "could not stat key");
-	succeed_if (keyNeedStat(key) == 1, "could not stat key");
-	keyDel (key);
-
 	key=0;
-	succeed_if (keyRemove (key) == -1, "remove null pointer");
-	succeed_if (keyNeedRemove(key) == -1, "could not remove key");
 	succeed_if (keyNeedSync(key) == -1, "key needs sync");
 	key = keyNew (0);
-	succeed_if (keyNeedRemove(key) == 0, "fresh key does not need remove");
 	succeed_if (keyNeedSync(key) == 0, "fresh key needs sync");
-	succeed_if (keyRemove(key) == 1, "could not remove key");
-	succeed_if (keyNeedRemove(key) == 1, "could not remove key");
-	succeed_if (keyNeedSync(key) == 1, "need sync after remove");
 	keyDel (key);
 
-	key = keyNew ("user/remove", KEY_REMOVE, KEY_END);
-	succeed_if (keyNeedRemove(key) == 1, "fresh key need remove");
-	succeed_if (keyNeedSync(key) == 1, "need sync after remove");
-	succeed_if (keyRemove(key) == 1, "could not remove key");
-	succeed_if (keyNeedRemove(key) == 1, "could not remove key");
-	succeed_if (keyNeedSync(key) == 1, "need sync after remove");
+	key = keyNew ("user/remove", KEY_END);
+	succeed_if (keyNeedSync(key) == 1, "need sync");
 	keyDel (key);
 
 	succeed_if (keyNeedSync(0) == -1, "keyNeedSync(0)");
-	succeed_if (keyNeedStat(0) == -1, "keyNeedStat(0)");
-	succeed_if (keyNeedRemove(0) == -1, "keyNeedRemove(0)");
 
 	key = keyNew(0);
 	succeed_if (keyNeedSync(key) == 0, "keyNew(0) should not need sync");
@@ -1915,13 +1788,13 @@ void test_keyMeta(void)
 	key = keyNew(0);
 	succeed_if(keyGetMode(key) == KEY_DEF_MODE, "new key does not have default mode");
 	succeed_if(keySetDir(key) == 0, "could not set dir");
-	succeed_if(keyGetMode(key) == KEY_DEF_MODE | KEY_DEF_DIR, "directory key");
+	succeed_if(keyGetMode(key) == (KEY_DEF_MODE | KEY_DEF_DIR), "directory key");
 	keyDel (key);
 
 	key = keyNew("user/dir", KEY_DIR, KEY_END);
-	succeed_if(keyGetMode(key) == KEY_DEF_MODE | KEY_DEF_DIR, "directory key");
+	succeed_if(keyGetMode(key) == (KEY_DEF_MODE | KEY_DEF_DIR), "directory key");
 	succeed_if(keySetDir(key) == 0, "could not set dir");
-	succeed_if(keyGetMode(key) == KEY_DEF_MODE | KEY_DEF_DIR, "directory key");
+	succeed_if(keyGetMode(key) == (KEY_DEF_MODE | KEY_DEF_DIR), "directory key");
 	keyDel (key);
 }
 
@@ -2221,18 +2094,214 @@ void test_keyNamespace()
 	keyDel (key);
 }
 
+typedef void (*fun_t) ();
+void fun()
+{}
 
-int main()
+void test_binary()
 {
-	printf("KEY  STRUCT  TESTS\n");
+	printf ("Test binary values\n");
+
+	Key *k = 0;
+
+	int i = 20;
+	int *p = &i;
+
+	k = keyNew(0);
+	succeed_if (keySetBinary(k, &p, sizeof(p)) == sizeof(p),
+			"could not set binary");
+
+	int *q;
+	succeed_if (keyGetBinary(k, &q, sizeof(q)) == sizeof(q),
+			"could not get binary");
+	succeed_if (p == q, "pointers to int are not equal");
+	succeed_if (*p == *q, "values are not equal");
+	succeed_if (*q == 20, "values are not equal");
+	succeed_if (*p == 20, "values are not equal");
+
+	keyDel (k);
+
+
+
+	union {void (*f)(); void* v;} conversation;
+
+	k = keyNew(0);
+	conversation.f = fun;
+	succeed_if (keySetBinary(k, &conversation.v, sizeof(conversation)) == sizeof(conversation),
+			"could not set binary");
+
+	conversation.v = 0;
+	conversation.f = 0;
+	void (*g) () = 0;
+	succeed_if (keyGetBinary(k, &conversation.v, sizeof(conversation)) == sizeof(conversation),
+			"could not get binary");
+	g = conversation.f;
+	succeed_if (g == fun, "pointers to functions are not equal");
+
+	keyDel (k);
+
+
+
+	conversation.f = fun;
+	k = keyNew("system/symbols/fun",
+			KEY_BINARY,
+			KEY_SIZE, sizeof (conversation),
+			KEY_VALUE, &conversation.v,
+			KEY_END);
+
+	conversation.v = 0;
+	conversation.f = 0;
+	succeed_if (keyGetBinary(k, &conversation.v, sizeof(conversation)) == sizeof(conversation),
+			"could not get binary");
+	g = conversation.f;
+	succeed_if (g == fun, "pointers to functions are not equal");
+
+	keyDel (k);
+
+
+
+
+	Plugin *plug = (Plugin *) 1222243;
+
+	k = 
+		keyNew("system/name",
+			KEY_BINARY,
+			KEY_SIZE, sizeof (plug),
+			KEY_VALUE, &plug,
+			KEY_END);
+	Plugin *xlug = *(Plugin**)keyValue(k);
+
+	succeed_if (xlug == plug, "should point to the same");
+	succeed_if (plug == (Plugin *) 1222243, "should point to that");
+	succeed_if (xlug == (Plugin *) 1222243, "should point to that too");
+
+	keyDel (k);
+
+
+	fun_t tmp = fun;
+
+	k = keyNew ("system/symbol/fun",
+			KEY_BINARY,
+			KEY_SIZE, sizeof (fun_t),
+			KEY_VALUE, &tmp,
+			KEY_END);
+
+	fun_t myfun = 0;
+	succeed_if (keyGetBinary(k, &myfun, sizeof(fun_t)) == sizeof(fun_t),
+			"could not get binary");
+
+	succeed_if (fun == myfun, "pointers not equal");
+
+	keyDel (k);
+
+
+	k = keyNew ("system/symbol/cool",
+			KEY_FUNC, fun,
+			KEY_END);
+
+	succeed_if (keyGetBinary(k, &myfun, sizeof(fun_t)) == sizeof(fun_t),
+			"could not get binary");
+
+	succeed_if (fun == myfun, "pointers not equal");
+
+	keyDel (k);
+}
+
+void test_outbreak()
+{
+	// TODO: outbreak bugs
+	Key *k = keyNew (KEY_END);
+
+	succeed_if (keySetName (k, "system/something/../..") == -1, "outbreak should not be allowed");
+	succeed_if (keySetName (k, "system/../something") == -1, "outbreak should not be allowed");
+
+	keySetName (k, "system/valid");
+	succeed_if (keySetBaseName (k, "..") == -1, "outbreak should not be allowed");
+
+	keySetName (k, "system/valid");
+	keyAddBaseName (k, "..");
+	succeed_if (keyAddBaseName (k, "..") == -1, "outbreak should not be allowed");
+
+	keyDel (k);
+}
+
+void test_keyBelowOrSame()
+{
+	Key * key1 = keyNew(0);
+	Key * key2 = keyNew(0);
+
+	printf("Test of keyBelowOrSame\n");
+
+	succeed_if (keyIsBelowOrSame(key1,0) == -1, "NULL pointer");
+	succeed_if (keyIsBelowOrSame(0,0) == -1, "NULL pointer");
+	succeed_if (keyIsBelowOrSame(0,key1) == -1, "NULL pointer");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (keyIsBelowOrSame(key2,key1), "Key should be below");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid/");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (keyIsBelowOrSame(key2,key1), "Key should be below");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid/below");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid/b");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid/b");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/valid");
+	keySetName(key2,"user/valid/b/e");
+	succeed_if (keyIsBelowOrSame(key1,key2), "Key should be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/valide");
+	keySetName(key2,"user/valid/e");
+	succeed_if (!keyIsBelowOrSame(key1,key2), "Key should not be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/valid/b");
+	keySetName(key2,"user/valid/e");
+	succeed_if (!keyIsBelowOrSame(key1,key2), "Key should not be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/export/a");
+	keySetName(key2,"user/export-backup/b");
+	succeed_if (!keyIsBelowOrSame(key1,key2), "Key should not be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keySetName(key1,"user/export");
+	keySetName(key2,"user/export-backup-2/x");
+	succeed_if (!keyIsBelowOrSame(key1,key2), "Key should not be below");
+	succeed_if (!keyIsBelowOrSame(key2,key1), "Key should not be below");
+
+	keyDel (key1);
+	keyDel (key2);
+}
+
+
+int main(int argc, char** argv)
+{
+	printf("    KEY  TESTS\n");
 	printf("==================\n\n");
 
-	init ();
+	init (argc, argv);
+
 	test_keyComparing();
 	test_keyNewSystem();
 	test_keyNewUser();
 	test_keyReference();
-	test_keyType();
 	test_keyName();
 	test_keyValue();
 	test_keyBinary();
@@ -2247,6 +2316,9 @@ int main()
 	test_keyMeta();
 	test_keyHelpers();
 	test_keyNamespace();
+	test_binary();
+	test_keyBelowOrSame();
+	// test_outbreak();
 
 	printf("\ntest_key RESULTS: %d test(s) done. %d error(s).\n", nbTest, nbError);
 
